using System;
using System.Collections.Generic;
using System.Text;

using Snak.CCNetScreenSaver.Core.State;
using System.Reflection;
using System.IO;

using ThoughtWorks.CruiseControl.Remote;
using Snak.CCNetScreenSaver.Properties;

namespace Snak.CCNetScreenSaver.Core
{
    internal class HtmlPageFactory
    {
        private FileInfo _settingsBackgroundFile = null;

        private FileInfo _buildStatusFile = null;
        
        internal FileInfo BuildStatusFile
        {
            get { return _buildStatusFile; }
        }

        internal HtmlPageFactory() { }

        internal string GetBuildStatusPageURL()
        {
            string url = String.Empty;

            if (Settings.Default.PageGenerationMode == PageGenerationMode.LocalGenerationOfHTML)
            {
                CreateBuildStatusPageHtmlLocally();
                url = _buildStatusFile.FullName;
            }
            else
            {
                url = GetExternalBuildStatusPageURL();
            }

            return url;
        }

        internal string GetSettingsPageURL()
        {
            if (_settingsBackgroundFile != null && _settingsBackgroundFile.Exists)
                _settingsBackgroundFile.Delete();

            _settingsBackgroundFile = new FileInfo(System.IO.Path.GetTempFileName() + ".htm");

            List<string> list = new List<string>();
            list.Add("Change Settings");

            File.WriteAllText(_settingsBackgroundFile.FullName, CreateHtml(list, IntegrationStatus.Unknown));

            return _settingsBackgroundFile.FullName;
        }

        private string GetExternalBuildStatusPageURL()
        {
            return Settings.Default.UrlOfExternalPageToDisplay.Replace("[[NAME]]", String.Join(",", ScreenSaverState.Instance.GetNamesToPublish().ToArray()).Replace("[[BUILD_STATUS]]", ScreenSaverState.Instance.IntegrationStatus.ToString()));
        }

        private void CreateBuildStatusPageHtmlLocally()
        {
            if (_buildStatusFile != null && _buildStatusFile.Exists)
                _buildStatusFile.Delete();

            _buildStatusFile = new FileInfo(System.IO.Path.GetTempFileName() + ".htm");

            File.WriteAllText(_buildStatusFile.FullName, CreateHtml(ScreenSaverState.Instance.GetNamesToPublish(), ScreenSaverState.Instance.IntegrationStatus));
        }

        private string CreateHtml(List<string> namesToPublish, IntegrationStatus integrationStatus)
        {
            StringBuilder message = new StringBuilder();
            string color = String.Empty;
            string name;

            for (int i = 0; i < namesToPublish.Count; i++)
            {
                // get rid of the domain name
                string[] userNameParts = namesToPublish[i].Split('\\');
                if (userNameParts.Length > 1)
                    name = userNameParts[1];
                else
                    name = userNameParts[0];

                message.Append("scObj0.block[");
                message.Append(i.ToString());
                message.Append(@"] = """);
                if ((integrationStatus == IntegrationStatus.Failure || integrationStatus == IntegrationStatus.Exception) && i == 0)
                {
                    message.Append("<span style='font-size:60%'>&lt;blame&gt;</span><br />");
                    message.Append(name);
                    message.Append("<br /><span style='font-size:60%'>&lt;/blame&gt;</span>");
                }
                else
                    message.Append(name);
                message.Append(@""";");
                message.Append(Environment.NewLine);
            }

            if (integrationStatus == IntegrationStatus.Failure || integrationStatus == IntegrationStatus.Exception)
                color = "red";
            else if (integrationStatus == IntegrationStatus.Success)
                color = "#00FF00";
            else
                color = "yellow";

            StringBuilder html = new StringBuilder();

            try
            {
                using (System.IO.Stream stream = Assembly.GetExecutingAssembly().GetManifestResourceStream("Snak.CCNetScreenSaver.Resources.Notice.htm"))
                {

                    using (StreamReader sr = new StreamReader(stream, Encoding.Default, true))
                    {
                        html.Append(sr.ReadToEnd());
                    }
                }
            }
            catch (Exception)
            {
                throw;
            }

            return html.Replace("[[MESSAGE]]", message.ToString()).Replace("[[BACKGROUNDCOLOR]]", color).ToString();
        }
    }
}
