using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Text;
using System.Reflection;

using Snak.CCNetScreenSaver.Core;


namespace Snak.CCNetScreenSaver.UI
{
    partial class ScreenSaverForm : Form
    {
        UIController _uiController = null;

        // Keep track of whether the screensaver has become active.
        private bool isActive = false;

        // Keep track of the location of the mouse
        private Point mouseLocation;

        public ScreenSaverForm(UIController uiController)
        {
            InitializeComponent();

            _uiController = uiController;
            
            SetupScreenSaver();
            this.Focus();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// As the browser is running in full screen any KeyDown events wired to the ScreenSaverForm dont get fired as the form does not have focus,
        /// the WebBrowser does not support the KeyDown event but the PreviewKeyDow seems close enough, however for some reason the PreviewKeyDown
        /// gets fired twice, not sure why hopefully someone can point it out to me. I've had to hack 
        /// around this, winforms really not my thing :( .... comments below
        /// 
        /// similar problem here: http://www.thescripts.com/forum/thread14255.html
        /// 
        /// these people think its a bug: https://forums.microsoft.com/MSDN/ShowPost.aspx?PostID=294572&SiteID=1
        /// 
        /// I must have changed something as its now only firing once. Rather than a bug I'd say its some multi threading 
        /// weird ness that I've somehow fixed as a results of fixing something else. I'll leave the comments here...
        /// </remarks>
        private bool _previewKeyDownMethodSkipOnce = false;
        void WebBrowser_PreviewKeyDown(object sender, PreviewKeyDownEventArgs e)
        {
            // HACK: as this event gets fired twice (see my 'remarks' comments in relation to this method) I've hacked together this bool
            // that get flipped each time this method gets called, so for every 2 time its called we return early once
            //if (_previewKeyDownMethodSkipOnce)
            //{
            //    _previewKeyDownMethodSkipOnce = false;
            //    return;
            //}

            if (e.KeyCode == Keys.S)
            {
                Cursor.Show();
                this.Capture = false;
                _uiController.ShowSettingsForm();
                SetMousePosition();
                this.Capture = true;
                Cursor.Hide();
                _previewKeyDownMethodSkipOnce = true;
            }
            else
            {
                _uiController.Stop();
            }
        }

        internal void NavigateBrowser(string url)
        {
            webBrowser.Navigate(url);
        }

        /// <summary>
        /// Set up the main form as a full screen screensaver.
        /// </summary>
        private void SetupScreenSaver()
        {
            // Use double buffering to improve drawing performance
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.UserPaint | ControlStyles.AllPaintingInWmPaint, true);
            // Capture the mouse
            this.Capture = true;

            // Set the application to full screen mode and hide the mouse
            Cursor.Hide();
            Bounds = Screen.PrimaryScreen.Bounds;
            WindowState = FormWindowState.Maximized;
            ShowInTaskbar = false;
            DoubleBuffered = true;
        }

        private void SetMousePosition()
        {
            mouseLocation = MousePosition;
            isActive = true;        
        }

        private void ScreenSaverForm_MouseMove(object sender, MouseEventArgs e)
        {
            // Set IsActive and MouseLocation only the first time this event is called.
            if (!isActive)
            {
                SetMousePosition();
            }
            else
            {
                // If the mouse has moved significantly since first call, close.
                if ((Math.Abs(MousePosition.X - mouseLocation.X) > 10) ||
                    (Math.Abs(MousePosition.Y - mouseLocation.Y) > 10))
                {
                    _uiController.Stop();
                }
            }
        }
    }
}
