using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Snak.CCNetScreenSaver.Core.State;

namespace Snak.CCNetScreenSaver.UI
{
    internal partial class SettingsForm : Form
    {
        private UIController _uiController = null;

        internal SettingsForm(UIController uiController)
        {
            InitializeComponent();

            _uiController = uiController;
            ShowInTaskbar = false;

            BindControls();
        }

        private void BindControls()
        {
            cruiseControlRemotingUriTextBox.Text = _uiController.CruiseControlRemotingInterfaceUri;
            cruiseControlRemotingUriTextBox.Text = _uiController.CruiseControlRemotingInterfaceUri;
            modificationsFileLocationPath.Text = _uiController.ModificationsFile;
            externalPageURLTextBox.Text = _uiController.UrlOfExternalPageToDisplay;
            pollingIntervalTextBox.Text = _uiController.PollingInterval.ToString();

            if (_uiController.PageGenerationMode == PageGenerationMode.RequestExternalHTML)
            {
                requestExternalHTMLRadioButton.Checked = true;
            }
            else
            {
                localGenerationOfHTMLRadioButton.Checked = true;
            }

            externalPageURLTextBox.Enabled = (_uiController.PageGenerationMode == PageGenerationMode.RequestExternalHTML);
            projectToMonitorComboBox.Enabled = _uiController.CruiseManagerOnLine;

            if (_uiController.CruiseManagerOnLine)
            {
                if (projectToMonitorComboBox.Items != null && projectToMonitorComboBox.Items.Count > 0)
                {
                    projectToMonitorComboBox.Items.Clear();
                    projectToMonitorComboBox.SelectedItem = null;
                }

                List<string> projects = _uiController.CCNetProjectsList;

                projectToMonitorComboBox.Items.Add("");

                if (projects.Count > 0)
                {
                    projectToMonitorComboBox.Items.AddRange(projects.ToArray());

                    if (projects.Contains(_uiController.ProjectToMonitor))
                        projectToMonitorComboBox.SelectedItem = _uiController.ProjectToMonitor;
                    else
                    {
                        projectToMonitorComboBox.SelectedItem = "";
                    }
                }
            }
        }

        private void SettingsForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
            {
                _uiController.HideSettingsForm();
            }
        }

        private void RefreshCCNetServerButton_Click(object sender, EventArgs e)
        {
            if (!_uiController.SetCCNetServer(cruiseControlRemotingUriTextBox.Text))
            {
                MessageBox.Show("Cruise control server not found", "Error", MessageBoxButtons.OK);
            }
            else
            {
                _uiController.CruiseControlRemotingInterfaceUri = cruiseControlRemotingUriTextBox.Text;
            }

            BindControls();
        }

        private void ModificationsFileDialogButton_Click(object sender, EventArgs e)
        {
            modificationsFileDialog.InitialDirectory = "c:\\" ;
            modificationsFileDialog.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*" ;
            modificationsFileDialog.FilterIndex = 2 ;
            modificationsFileDialog.RestoreDirectory = true ;

            if(modificationsFileDialog.ShowDialog() == DialogResult.OK)
            {
                modificationsFileLocationPath.Text = modificationsFileDialog.FileName;
            }

            modificationsFileLocationPath.Focus();
        }

        private void SaveSettingsButton_Click(object sender, EventArgs e)
        {
            _uiController.SaveSettings();
        }

        private void ModeChanged_CheckedChanged(object sender, EventArgs e)
        {
            if (requestExternalHTMLRadioButton.Checked)
            {
                _uiController.PageGenerationMode = PageGenerationMode.RequestExternalHTML;
            }
            else
            {
                _uiController.PageGenerationMode = PageGenerationMode.LocalGenerationOfHTML;
            }

            _uiController.UrlOfExternalPageToDisplay = externalPageURLTextBox.Text;

            externalPageURLTextBox.Enabled = (_uiController.PageGenerationMode == PageGenerationMode.RequestExternalHTML);
            // BindControls();
        }

        private void CruiseControlRemotingUriTextBox_Leave(object sender, EventArgs e)
        {
            _uiController.CruiseControlRemotingInterfaceUri = cruiseControlRemotingUriTextBox.Text;
        }

        private void ProjectToMonitorComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            _uiController.ProjectToMonitor = (projectToMonitorComboBox.SelectedItem == null) ? String.Empty : projectToMonitorComboBox.SelectedItem.ToString();
        }

        private void ModificationsFileLocationPath_Leave(object sender, EventArgs e)
        {
            _uiController.ModificationsFile = modificationsFileLocationPath.Text;
        }

        private void ExternalPageURLTextBox_Leave(object sender, EventArgs e)
        {
            _uiController.UrlOfExternalPageToDisplay = externalPageURLTextBox.Text;
        }

        private void PollingIntervalTextBox_Leave(object sender, EventArgs e)
        {
            int value;
            int.TryParse(pollingIntervalTextBox.Text, out value);

            _uiController.PollingInterval = value;
        }

        private void SettingsForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            _uiController.HideSettingsForm();
        }

        private void ResetSettingsButton_Click(object sender, EventArgs e)
        {
            _uiController.ResetSettings();
            BindControls();
        }
    }
}