using System;
using System.Collections.Generic;
using System.Text;
using Snak.CCNetScreenSaver.Core.CCNet;
using Snak.CCNetScreenSaver.Core.State;
using Snak.CCNetScreenSaver.Properties;
using Snak.CCNetScreenSaver.Core;
using System.Windows.Forms;

namespace Snak.CCNetScreenSaver.UI
{
    internal class UIController
    {
        private ScreenSaverForm _screenSaverForm = null;
        private SettingsForm _settingsForm = null;
        private HtmlPageFactory _htmlPageFactory = new HtmlPageFactory();

        internal string CruiseControlRemotingInterfaceUri
        {
            get 
            {
                return ScreenSaverState.Instance.CruiseControlRemotingInterfaceUri;
            }
            set 
            {
                ScreenSaverState.Instance.CruiseControlRemotingInterfaceUri = value;
            }
        }

        internal string ProjectToMonitor
        {
            get
            {
                return ScreenSaverState.Instance.ProjectToMonitor;
            }
            set
            {
                ScreenSaverState.Instance.ProjectToMonitor = value;
            }
        }

        internal string ModificationsFile
        {
            get
            {
                return ScreenSaverState.Instance.ModificationsFile;
            }
            set
            {
                ScreenSaverState.Instance.ModificationsFile = value;
            }
        }

        internal string UrlOfExternalPageToDisplay
        {
            get
            {
                return ScreenSaverState.Instance.UrlOfExternalPageToDisplay;
            }
            set
            {
                ScreenSaverState.Instance.UrlOfExternalPageToDisplay = value;
            }
        }

        internal int PollingInterval
        {
            get
            {
                return ScreenSaverState.Instance.PollingInterval;
            }
            set
            {
                ScreenSaverState.Instance.PollingInterval = value;
            }
        }

        internal PageGenerationMode PageGenerationMode
        {
            get
            {
                return ScreenSaverState.Instance.PageGenerationMode;
            }
            set
            {
                ScreenSaverState.Instance.PageGenerationMode = value;
            }
        }

        internal bool CruiseManagerOnLine
        {
            get 
            {
                return CCNetFacade.Instance.CruiseManagerOnLine;
            }
        }

        internal List<string> CCNetProjectsList
        {
            get
            {
                return CCNetFacade.Instance.GetCCNetProjectsList();
            }
        }



        internal UIController()
        {
            ScreenSaverState.Instance.OnIntegrationStatusChanged += new EventHandler(OnIntegrationStatusChange);
        }

        internal void Start()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            _screenSaverForm = new ScreenSaverForm(this);

            CCNetFacade.Instance.StartPollingBuildServer();

            Application.Run(_screenSaverForm);
        }

        internal void Stop()
        {
            if (_settingsForm != null)
            {
                _settingsForm.Close();
                _settingsForm.Dispose();
            }
            if (_screenSaverForm != null)
            {
                _screenSaverForm.Close();
                _screenSaverForm.Dispose();
            }
        }

        internal void OnIntegrationStatusChange(object sender, EventArgs e)
        {
            string url = _htmlPageFactory.GetBuildStatusPageURL();
            _screenSaverForm.NavigateBrowser(url);
        }

        internal void SaveSettings()
        {
            ScreenSaverState.Instance.Save();
        }

        internal void ShowSettingsForm()
        {
            CCNetFacade.Instance.StopPollingBuildServer();

            string settignsUrl = _htmlPageFactory.GetSettingsPageURL();
            _screenSaverForm.NavigateBrowser(settignsUrl);

            _settingsForm = new SettingsForm(this);
            _settingsForm.ShowDialog(_screenSaverForm);
        }

        internal void HideSettingsForm()
        {
            // if the user has asked to close the setting form but hasn't explicitly clicked 
            // the save button then we will roll back any pending confg changes.
            ScreenSaverState.Instance.DiscardPendingChangesIfAny();
            // load up any new changes
            ScreenSaverState.Instance.Load();

            if (_settingsForm != null)
            {
                _settingsForm.Hide();
                _settingsForm.Dispose();
            }

            CCNetFacade.Instance.StartPollingBuildServer();
        }

        internal bool SetCCNetServer(string uri)
        {
            return CCNetFacade.Instance.SetCCNetServer(uri);
        }

        internal void ResetSettings()
        {
            ScreenSaverState.Instance.Reset();
        }
    }
}
