using System;
using System.Xml;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Configuration;

using NAnt.VSNet.Types;
using Tests.NAnt.Contrib;
using NUnit.Framework;

using Snak.Core;
using Snak.BuildDrop;
using System.Diagnostics;

namespace Snak.FunctionalTests
{
    /// <summary>
    ///  
    /// </summary>
    /// <remarks>
    ///  This class has obvious dependencies upon other parts of the SNAK library (e.g. it uses BuildDrop1, ISolutionInfo). As such it assumes that these
    ///  are working correctly. As we are only testing the DeploySolutionTask this should not be an issue however if BuildDrop1 or ISolutionInfo are broken that
    ///  will have obvious effects here. Its not that bad utilizing theses classes because we at least gain more code coverage with our tests.
    /// </remarks>
    public abstract class AbstractDeploySolutionTaskTest : AbstractBuildRunnerTester
    {
        public abstract override Dictionary<string, WebMap> WebMaps { get; }

        public AbstractDeploySolutionTaskTest(string relativePathtoSolution, string relativePathtoDropDir, DotNetFrameworkVersion dotNetFrameworkVersion)
            : base(relativePathtoSolution, relativePathtoDropDir, dotNetFrameworkVersion)
        {
            
        }

        /// <summary>
        /// build the solution passed to the constructor of this class, deploys the output, checks the deployment, if createVirtualDirsForWebsProjects is true
        /// creates virtual directories for web apps and checks that they work.
        /// </summary>
        /// <remarks></remarks>
        /// <param name="buildConfig"></param>
        /// <param name="createVirtualDirsForWebsProjects"></param>
        public void BuildDeployAndCheckDeployment(string testName, string buildConfig, bool createVirtualDirsForWebsProjects)
        {
            BuildDeployAndCheckDeployment(testName, buildConfig, String.Empty, createVirtualDirsForWebsProjects);
        }

        /// <summary>
        /// build the solution passed to the constructor of this class, deploys the output, checks the deployment, if createVirtualDirsForWebsProjects is true
        /// creates virtual directories for web apps and checks that they work.
        /// </summary>
        /// <remarks></remarks>
        /// <param name="buildConfiguration"></param>
        /// <param name="platform"></param>
        /// <param name="createVirtualDirsForWebsProjects"></param>
        public void BuildDeployAndCheckDeployment(string testName, string buildConfiguration, string platform, bool createVirtualDirsForWebsProjects)
        {
            try
            {
                string projectXml = CreateProjetXmlSnippet(buildConfiguration, platform, createVirtualDirsForWebsProjects, testName);

                IDropNamingStrategy packageNamingStrategy = new CommonDirectoryDropNamingStrategy(this.Solution.SolutionFile.Name, this.DropDir, new BuildVersion("65"), GetCorrectBuildConfigString(buildConfiguration, platform));
                BuildDrop.BuildDrop buildDrop = new BuildDrop.BuildDrop(packageNamingStrategy, this.Solution, GetCorrectBuildConfigString(buildConfiguration, platform));
                // delete anything that exists from the last test run
                buildDrop.DeleteDropDirectores();

                string coneoleOutput = RunBuild(projectXml);

                if (!CheckDeployedFilesExistInBothCommonAndCurrentDropDirectories(buildDrop))
                {
                    Assert.Fail("Error on deployment. Either no files were deployed to the target folders or else the labeled drop folder does not match the 'current' drop folder.");
                }

                // if we deployed the web projects we will make a request to the web app in the project and see that we 
                // get results, its a bit hacky but should be sufficient. We will parse the page and check that a given string exists 
                // just to confirm that we didnt receive a server error 
                if (createVirtualDirsForWebsProjects)
                {
                    System.Net.WebClient browser = new System.Net.WebClient();
                    browser.UseDefaultCredentials = true;

                    string url = "http://localhost/SnakFunctionalTests_WebApplication1_ANiceSuffix/Default.aspx";

                    if (VirDirServerName != null && VirDirServerName != String.Empty)
                        url = "http://" + VirDirServerName + "/SnakFunctionalTests_WebApplication1_ANiceSuffix/Default.aspx";

                    string page = browser.DownloadString(url);

                    if (page == null || page == String.Empty || !page.Contains("It works!"))
                        Assert.Fail("Error accessing the web project at " + page + ". At the time of the test this is expected to map to the WebApplication1 project that belongs to the '" + this.Solution.SolutionFile.FullName + "' solution");

                    // delete any virtual directories what were created for the tests
                    // TODO: 
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                throw;
            }
        }

        /// <summary>
        /// creates an XML string useful to pass to nant to test the DeploySolutionTask, uses both parameters and internal 
        /// class data to determine the structure of the xml.
        /// </summary>
        /// <remarks>ekk. this method is getting a bit big! however its fairly flat in that it doesnt call out to other functionality, so shouldnt be too hard to understand</remarks>
        /// <param name="buildConfiguration"></param>
        /// <param name="platform"></param>
        /// <param name="createVirtualDirsForWebsProjects"></param>
        /// <returns></returns>
        protected string CreateProjetXmlSnippet(string buildConfiguration, string platform, bool createVirtualDirsForWebsProjects, string testName)
        {
            StringBuilder solutionXMLSnippet = new StringBuilder();
            StringBuilder webMappings = new StringBuilder();
            StringBuilder virtualDirectorySettings = new StringBuilder();
            IProjectInfo[] projects = this.Solution.GetProjectsFor(GetCorrectBuildConfigString(buildConfiguration, platform));

            string buildTask = String.Empty;

            if (this.DotNetFrameworkVersion == DotNetFrameworkVersion.v1_1)
            {
                solutionXMLSnippet.Append(@"<solution configuration=""");
                solutionXMLSnippet.Append(buildConfiguration);
                solutionXMLSnippet.Append(@""" solutionfile=""");
                solutionXMLSnippet.Append(this.SolutionFile.FullName);
                solutionXMLSnippet.Append(@""">");

                webMappings.Append(@"<webmap>");

                foreach (IProjectInfo project in projects)
                {
                    if (project.IsWebProject)
                    {
                        webMappings.Append(@"<map url=""");
                        webMappings.Append(this.WebMaps[project.Name].Url);
                        webMappings.Append(@""" path=""");
                        webMappings.Append(this.WebMaps[project.Name].Path);
                        webMappings.Append(@""" />");
                    }
                }

                webMappings.Append(@"</webmap>");
                solutionXMLSnippet.Append(webMappings.ToString());
                solutionXMLSnippet.Append("</solution>");

                buildTask = solutionXMLSnippet.ToString();
            }
            else if (this.DotNetFrameworkVersion == DotNetFrameworkVersion.v2_0)
            {
                // /p:Configuration=Debug /p:Platform="Any CPU"
                buildTask = @"  <msbuild project=""" + this.SolutionFile.FullName + @""">
                                    <property name=""Configuration"" value=""" + buildConfiguration + @""" />
                                    <property name=""Platform"" value=""" + platform + @""" />
                                </msbuild>";
            }

            if (createVirtualDirsForWebsProjects)
            {
                virtualDirectorySettings.Append(@"<virDirSettings>");

                // figure out the virtual directory settings
                foreach (IProjectInfo project in projects)
                {
                    if (project.IsWebProject)
                    {
                        virtualDirectorySettings.Append(@"<virDirSetting projectName=""");
                        virtualDirectorySettings.Append(project.Name);
                        virtualDirectorySettings.Append(@""" clrVersion=""");
                        virtualDirectorySettings.Append((project.DotNetFrameworkVersion == DotNetFrameworkVersion.v1_1) ? "v1.1.4322" : "v2.0.50727"); // bit of a hardcoded dependency here... its not that bad
                        virtualDirectorySettings.Append(@""" authanonymous=""");
                        virtualDirectorySettings.Append("True");
                        virtualDirectorySettings.Append(@""" authbasic=""");
                        virtualDirectorySettings.Append("False");
                        virtualDirectorySettings.Append(@""" authntlm=""");
                        virtualDirectorySettings.Append("True");
                        virtualDirectorySettings.Append(@""" enabledirbrowsing=""");
                        virtualDirectorySettings.Append("False");
                        virtualDirectorySettings.Append(@""" iisserver=""");
                        virtualDirectorySettings.Append(this.VirDirServerName);                        
                        virtualDirectorySettings.Append(@""" defaultdoc=""");
                        virtualDirectorySettings.Append("Default.htm,Default.asp,index.htm,iisstart.asp,Default.aspx");
                        virtualDirectorySettings.Append(@""" />");
                    }
                }

                virtualDirectorySettings.Append(@"</virDirSettings>");
            }

            string xmlsnippet = String.Format(
                                    @"<project name=""myTestProject"">
                                        {0}
                                        <loadtasks assembly=""{1}"" />
                                        {2}
                                        <deploySolution 
                                          solution=""{3}"" 
                                          config=""{4}"" 
                                          artifactsDir=""{5}""
                                          buildLabel=""65""
                                          webProjects_SetUpVirtualDirectories=""{6}""
                                          webProjects_VirtualDirectoriesPrefix=""SnakFunctionalTests_""
                                          webProjects_VirtualDirectoriesSuffix=""_ANiceSuffix""
                                          property=""deploySolutionTask""
                                          >
                                          {7}
                                          {8}
                                        </deploySolution>
                                      </project>",
                                    // tell nant which .net framework to compile with
                                    ((this.DotNetFrameworkVersion == DotNetFrameworkVersion.v1_1) ? @"<property name=""nant.settings.currentframework"" value=""net-1.1"" />" : @"<property name=""nant.settings.currentframework"" value=""net-2.0"" />"),
                                    // GetFullPath will return the rooted path to where ever this the test is running from, as it turns out 
                                    // this is different from where the base class writes this xml and loads it into nant, hence we need the full path to Snak.dll
                                    Path.GetFullPath("Snak.dll"),
                                    buildTask,
                                    this.SolutionFile.FullName,
                                    this.GetCorrectBuildConfigString(buildConfiguration, platform),
                                    this.DropDir.FullName,
                                    createVirtualDirsForWebsProjects.ToString(),
                                    // specify the web mapping similar to those passed to the 'solution' task that comes out of the box with nant
                                    webMappings.ToString(),
                                    // if we are setting up virtual directories for web projects we need to specify the setting for them
                                    // check out the doco in DeploySolutionTask for more info and an xml example
                                    ((createVirtualDirsForWebsProjects) ? virtualDirectorySettings.ToString() : String.Empty)
                                );

            // do we want to write the temp project file for later use (e.g. as an example reference)
            if (Snak.FunctionalTests.Properties.Settings.Default.WriteDynamicallyCreatedBuildFilesToDisk)
            {
                WriteBuildFileToDisk(testName, xmlsnippet);
            }

            return xmlsnippet;
        }
    }
}

