using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

using NAnt.Core.Types;
using NAnt.VSNet.Types;
using Tests.NAnt.Contrib;

using NUnit.Framework;

using Snak.Core;
using Snak.BuildDrop;
using System.Reflection;

namespace Snak.FunctionalTests
{
    [TestFixture]
    public class CleanTaskTests : AbstractDeploySolutionTaskTest
    {
        public override Dictionary<string, WebMap> WebMaps
        {
            get
            {
                // WebMaps not used in 2.0 web projects
                return null;
            }
        }

        public CleanTaskTests()
            : base
            (
                @"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\SolutionWithMultipleDifferingProjects.sln",
                @"..\..\..\TestArtifacts\BuildDrops\v2.0\C#\SolutionWithMultipleDifferingProjects\",
                DotNetFrameworkVersion.v2_0
            ) { }

        [Test]
        public void CleanBySpecifyingASolution()
        {
            BuildDeployAndCheckDeployment(MethodInfo.GetCurrentMethod().Name, "Debug", "Any CPU", false);

            string xmlsnippet = String.Format(
                                    @"  <project name=""cleanTaksTestProject"">
                                            <loadtasks assembly=""{0}"" />
                                            <clean solution=""{1}"" config=""{2}""/>	
                                        </project>",
                                        Path.GetFullPath("Snak.dll"),
                                        this.SolutionFile.FullName,
                                        "Debug|Any CPU"
                                    );

            RunBuild(xmlsnippet);

            // make sure the files in the working directory (as captured above before we ran the nant script) have been deleted
            foreach (IProjectInfo project in this.Solution.GetProjectsFor("Debug|Any CPU"))
            {
                if (new DirectoryInfo(project.OutputPathFull).Exists)
                    Assert.Fail("The output directory '{0}' was spoused to have been deleted.", project.OutputPathFull);
            }
        }

        [Test]
        public void CleanBySpecifyingAFileSet()
        {
            BuildDeployAndCheckDeployment(MethodInfo.GetCurrentMethod().Name, "Debug", "Any CPU", false);

            string filesInObjDirToDelete = Path.GetFullPath(@"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\") + @"**\obj\**";
            string filesInBinDirToDelete = Path.GetFullPath(@"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\") + @"**\bin\**";

            FileSet filesToDelete = new FileSet();
            filesToDelete.Includes.Add(filesInObjDirToDelete);
            filesToDelete.Includes.Add(filesInBinDirToDelete);

            string xmlsnippet = String.Format(
                                    @"  <project name=""cleanTaksTestProject"">
                                            <loadtasks assembly=""{0}"" />
                                            <clean>
                                                <items>
                                                    <include name=""{2}""/>
                                                    <include name=""{3}""/>
                                                </items>			
                                            </clean>	
                                        </project>",
                                        Path.GetFullPath("Snak.dll"),
                                        this.DropDir.FullName,
                                        filesInObjDirToDelete,
                                        filesInBinDirToDelete
                                    );

            RunBuild(xmlsnippet);

            // make sure the files in the working directory (as captured above before we ran the nant script) have been deleted
            foreach (string fileName in filesToDelete.FileNames)
            {
                if (new FileInfo(fileName).Exists)
                    Assert.Fail("The file '{0}' was spoused to have been deleted.", fileName);

                Console.WriteLine("Successful deletion of the file: {0}", fileName);
            }
        }
    }
}