using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

using NAnt.VSNet.Types;
using Tests.NAnt.Contrib;
using NUnit.Framework;

namespace Snak.FunctionalTests
{
    [TestFixture, Category("IntegrationTest")]
    public class GetFirstDirectoryNameBasedOnSortTaskTests : BuildTestBase
    {
        [Test]
        public void SortDescending()
        {
            string xmlsnippet = String.Format(
                                    @"<project name=""getFirstDirectoryNameBasedOnSort"">
                                        <loadtasks assembly=""{0}"" />
		                                <getFirstDirectoryNameBasedOnSort 
			                                pathToContainingFolder=""{1}"" 
			                                sortDescending=""true"" 
			                                directoryNameFilter=""windowsservice1_Release-AnyCPU_*""
			                                property=""latestWindowsService1ReleaseBuildDirectory""
			                                defaultValue=""C:\someDirectory\latestWindowsService1ReleaseBuildDirectory\""
		                                />	
                                        <fail unless=""${{property::exists('latestWindowsService1ReleaseBuildDirectory.FullPathToFirstDirectory')}}"">#1</fail>
                                        <fail if=""${{latestWindowsService1ReleaseBuildDirectory.FullPathToFirstDirectory != '{2}'}}"">#2</fail>
                                        <echo message=""First directory when sorting descending was: ${{latestWindowsService1ReleaseBuildDirectory.FullPathToFirstDirectory}}"" />
                                      </project>",
                                    Path.GetFullPath("Snak.dll"),
                                    Path.GetFullPath(@"..\..\..\TestArtifacts\GetFirstDirectoryNameBasedOnSortArtifacts\"),
                                    Path.GetFullPath(@"..\..\..\TestArtifacts\GetFirstDirectoryNameBasedOnSortArtifacts\windowsservice1_Release-AnyCPU_65")
                                );
            
            RunBuild(xmlsnippet);
        }

        [Test]
        public void SortAscending()
        {
            string xmlsnippet = String.Format(
                                    @"<project name=""getFirstDirectoryNameBasedOnSort"">
                                        <loadtasks assembly=""{0}"" />
		                                <getFirstDirectoryNameBasedOnSort 
			                                pathToContainingFolder=""{1}"" 
			                                directoryNameFilter=""windowsservice1_Release-AnyCPU_*""
			                                property=""latestWindowsService1ReleaseBuildDirectory""
			                                defaultValue=""C:\someDirectory\latestWindowsService1ReleaseBuildDirectory\""
		                                />	
                                        <fail unless=""${{property::exists('latestWindowsService1ReleaseBuildDirectory.FullPathToFirstDirectory')}}"">#1</fail>
                                        <fail if=""${{latestWindowsService1ReleaseBuildDirectory.FullPathToFirstDirectory != '{2}'}}"">#2</fail>
                                        <echo message=""First directory when sorting assending was: ${{latestWindowsService1ReleaseBuildDirectory.FullPathToFirstDirectory}}"" />
                                      </project>",
                                    Path.GetFullPath("Snak.dll"),
                                    Path.GetFullPath(@"..\..\..\TestArtifacts\GetFirstDirectoryNameBasedOnSortArtifacts\"),
                                    Path.GetFullPath(@"..\..\..\TestArtifacts\GetFirstDirectoryNameBasedOnSortArtifacts\windowsservice1_Release-AnyCPU_59")
                                );

            RunBuild(xmlsnippet);
        }

        [Test]
        public void CheckUseDefaultValue()
        {
            string xmlsnippet = String.Format(
                                    @"<project name=""getFirstDirectoryNameBasedOnSort"">
                                        <loadtasks assembly=""{0}"" />
		                                <getFirstDirectoryNameBasedOnSort 
			                                pathToContainingFolder=""{1}"" 
			                                directoryNameFilter=""windowsservice1_Release-AnyCPU_NoDirectoryHere_*""
			                                property=""latestWindowsService1ReleaseBuildDirectory""
			                                defaultValue=""C:\someDirectory\latestWindowsService1ReleaseBuildDirectory""
		                                />	
                                        <fail unless=""${{property::exists('latestWindowsService1ReleaseBuildDirectory.FullPathToFirstDirectory')}}"">#1</fail>
                                        <fail if=""${{latestWindowsService1ReleaseBuildDirectory.FullPathToFirstDirectory != '{2}'}}"">#2</fail>
                                      </project>",
                                    Path.GetFullPath("Snak.dll"),
                                    Path.GetFullPath(@"..\..\..\TestArtifacts\GetFirstDirectoryNameBasedOnSortArtifacts\"),
                                    Path.GetFullPath(@"C:\someDirectory\latestWindowsService1ReleaseBuildDirectory")
                                );

            RunBuild(xmlsnippet);
        }
    }
}
