using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using Tests.NAnt.Contrib;
using System.IO;

namespace Snak.FunctionalTests
{
    [TestFixture]
    public class LoadVersionTaskTests : BuildTestBase
    {
        delegate string GetXml(string versionAttribute);

        [Test]
        public void LoadVersion()
        {
            try
            {
                string tempVersionFile = Path.GetTempFileName();
                File.WriteAllText(tempVersionFile, "2.4.34.1");

                Delegate getXml = new GetXml(
                    delegate(string versionAttribute)
                    {
                        return String.Format(
                                            @"  <project name=""loadVersionTestsProject"">
                                                    <loadtasks assembly=""{0}"" />
                                                    <loadVersion
                                                        {1}
                                                        property=""loadVersionProperty""
                                                    />
                                                    <fail unless=""${{property::exists('loadVersionProperty.version.major')}}"">#1</fail>
                                                    <fail unless=""${{property::exists('loadVersionProperty.version.minor')}}"">#2</fail>
                                                    <fail unless=""${{property::exists('loadVersionProperty.version.build')}}"">#3</fail>
                                                    <fail unless=""${{property::exists('loadVersionProperty.version.revision')}}"">#4</fail>
                                                    <fail unless=""${{property::exists('loadVersionProperty.version.path')}}"">#5</fail>

                                                    <fail unless=""${{loadVersionProperty.version.major == '2'}}"">#6</fail>
                                                    <fail unless=""${{loadVersionProperty.version.minor == '4'}}"">#7</fail>
                                                    <fail unless=""${{loadVersionProperty.version.build == '34'}}"">#8</fail>
                                                    <fail unless=""${{loadVersionProperty.version.revision == '1'}}"">#9</fail>
                                                    <fail unless=""${{loadVersionProperty.version.path == '2_4_34_1'}}"">#10</fail>

                                                    <fail unless=""${{property::exists('loadVersionProperty.version')}}"">#11</fail>
                                                    <fail unless=""${{loadVersionProperty.version == '2.4.34.1'}}"">#12</fail>
                                                </project>",
                                            Path.GetFullPath("Snak.dll"),
                                            versionAttribute
                                        );
                    });

                try
                {
                    string xmlsnippet = (string)getXml.DynamicInvoke(new object[] { "versionFile=\"" + tempVersionFile + "\"" });

                    RunBuild(xmlsnippet);

                    xmlsnippet = (string)getXml.DynamicInvoke(new object[] { "versionString=\"2.4.34.1\"" });

                    RunBuild(xmlsnippet);
                }
                finally
                {
                    File.Delete(tempVersionFile);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                throw;
            }
        }

        [Test]
        public void LoadVersionWithOverRides()
        {
            try
            {
                string tempVersionFile = Path.GetTempFileName();
                File.WriteAllText(tempVersionFile, "2.4.34.1");

                Delegate getXml = new GetXml(
                    delegate(string versionAttribute)
                    {
                        return String.Format(
                                            @"  <project name=""loadVersionTestsProject"">
                                                    <loadtasks assembly=""{0}"" />
                                                    <loadVersion
                                                        {1}
                                                        property=""loadVersionProperty""
                                                        overrideMajorValue=""1""
                                                        overrideMinorValue=""2""
                                                        overrideBuildValue=""3""
                                                        overrideRevisionValue=""4""
                                                    />
                                                    <fail unless=""${{property::exists('loadVersionProperty.version.major')}}"">#1</fail>
                                                    <fail unless=""${{property::exists('loadVersionProperty.version.minor')}}"">#2</fail>
                                                    <fail unless=""${{property::exists('loadVersionProperty.version.build')}}"">#3</fail>
                                                    <fail unless=""${{property::exists('loadVersionProperty.version.revision')}}"">#4</fail>
                                                    <fail unless=""${{property::exists('loadVersionProperty.version.path')}}"">#5</fail>

                                                    <fail unless=""${{loadVersionProperty.version.major == '1'}}"">#6</fail>
                                                    <fail unless=""${{loadVersionProperty.version.minor == '2'}}"">#7</fail>
                                                    <fail unless=""${{loadVersionProperty.version.build == '3'}}"">#8</fail>
                                                    <fail unless=""${{loadVersionProperty.version.revision == '4'}}"">#9</fail>
                                                    <fail unless=""${{loadVersionProperty.version.path == '1_2_3_4'}}"">#10</fail>

                                                    <fail unless=""${{property::exists('loadVersionProperty.version')}}"">#11</fail>
                                                    <fail unless=""${{loadVersionProperty.version == '1.2.3.4'}}"">#12</fail>
                                                </project>",
                                            Path.GetFullPath("Snak.dll"),
                                            versionAttribute
                                        );
                    });

                try
                {
                    string xmlsnippet = (string)getXml.DynamicInvoke(new object[] { "versionFile=\"" + tempVersionFile + "\"" });

                    RunBuild(xmlsnippet);

                    xmlsnippet = (string)getXml.DynamicInvoke(new object[] { "versionString=\"2.4.34.1\"" });
                  
                    RunBuild(xmlsnippet);
                }
                finally
                {
                    File.Delete(tempVersionFile);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);
                throw;
            }
        }
    }
}
