using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using System.Reflection;
using NAnt.VSNet.Types;
using Snak.Core;
using System.IO;

namespace Snak.FunctionalTests
{
    public class PackageOutputsTaskTests
    {
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //
        // .Net v1.1 deployment tests (Everett)
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        [TestFixture]
        public class EverettCSharpeDeploymentTests : AbstractPackageOutputsTaskTests
        {
            private Dictionary<string, WebMap> _webMaps = null;

            public override Dictionary<string, WebMap> WebMaps
            {
                get
                {
                    if (_webMaps == null)
                    {
                        _webMaps = new Dictionary<string, WebMap>();
                        WebMap map = new WebMap();
                        map.Url = "http://localhost/WebApplication1/WebApplication1.csproj";
                        map.Path = new FileInfo(Path.GetFullPath(@"..\..\..\TestArtifacts\SampleApplications\v1.1\C#\AspNetWebApplication\WebApplication1\WebApplication1.csproj"));
                        _webMaps.Add("WebApplication1", map);
                        map = new WebMap();
                        map.Url = "http://localhost/WebService1/WebService1.csproj";
                        map.Path = new FileInfo(Path.GetFullPath(@"..\..\..\TestArtifacts\SampleApplications\v1.1\C#\AspNetWebApplication\WebService1\WebService1.csproj"));
                        _webMaps.Add("WebService1", map);
                    }

                    return _webMaps;
                }
            }

            public EverettCSharpeDeploymentTests()
                : base
                (
                    @"..\..\..\TestArtifacts\SampleApplications\v1.1\C#\SolutionWithMultipleDifferingProjects.sln",
                    @"..\..\..\TestArtifacts\BuildDrops\v1.1\C#\SolutionWithMultipleDifferingProjects\",
                    DotNetFrameworkVersion.v1_1
                ) { }

            [Test]
            public void DeployMutipleProjects_Debug()
            {
                BuildDeployAndCheckDeployment(MethodInfo.GetCurrentMethod().Name, "Debug");
            }

            [Test]
            public void DeployMutipleProjects_Release()
            {
                BuildDeployAndCheckDeployment(MethodInfo.GetCurrentMethod().Name, "Release");
            }
        }

        [TestFixture]
        public class EverettVBDeploymentTests : AbstractPackageOutputsTaskTests
        {
            private Dictionary<string, WebMap> _webMaps = null;

            public override Dictionary<string, WebMap> WebMaps
            {
                get
                {
                    if (_webMaps == null)
                    {
                        _webMaps = new Dictionary<string, WebMap>();
                        WebMap map = new WebMap();
                        map.Url = "http://localhost/WebApplication1/WebApplication1.vbproj";
                        map.Path = new FileInfo(Path.GetFullPath(@"..\..\..\TestArtifacts\SampleApplications\v1.1\VB\AspNetWebApplication\WebApplication1\WebApplication1.vbproj"));
                        _webMaps.Add("WebApplication1", map);
                        map = new WebMap();
                        map.Url = "http://localhost/WebService1/WebService1.vbproj";
                        map.Path = new FileInfo(Path.GetFullPath(@"..\..\..\TestArtifacts\SampleApplications\v1.1\VB\AspNetWebApplication\WebService1\WebService1.vbproj"));
                        _webMaps.Add("WebService1", map);
                    }

                    return _webMaps;
                }
            }

            public EverettVBDeploymentTests()
                : base
                (
                    @"..\..\..\TestArtifacts\SampleApplications\v1.1\VB\SolutionWithMultipleDifferingProjects.sln",
                    @"..\..\..\TestArtifacts\BuildDrops\v1.1\VB\SolutionWithMultipleDifferingProjects\",
                    DotNetFrameworkVersion.v1_1
                ) { }

            [Test]
            public void DeployMutipleProjects_Debug()
            {
                BuildDeployAndCheckDeployment(MethodInfo.GetCurrentMethod().Name, "Debug");
            }

            [Test]
            public void DeployMutipleProjects_Release()
            {
                BuildDeployAndCheckDeployment(MethodInfo.GetCurrentMethod().Name, "Release");
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //
        // .Net v2.0 deployment tests (Whidbey)
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        [TestFixture]
        public class WhidbeyCSharpeDeploymentTests : AbstractPackageOutputsTaskTests
        {
            public override Dictionary<string, WebMap> WebMaps
            {
                get
                {
                    // WebMaps not used in 2.0 web projects
                    return null;
                }
            }

            public override string VirDirServerName
            {
                get
                {
                    return @"localhost:8099";
                }
            }

            public WhidbeyCSharpeDeploymentTests()
                : base
                (
                    @"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\SolutionWithMultipleDifferingProjects.sln",
                    @"..\..\..\TestArtifacts\BuildDrops\v2.0\C#\SolutionWithMultipleDifferingProjects\",
                    DotNetFrameworkVersion.v2_0
                ) { }

            [Test]
            public void DeployMutipleProjects_Debug()
            {
                BuildDeployAndCheckDeployment(MethodInfo.GetCurrentMethod().Name, "Debug", "Any CPU");
            }

            [Test]
            public void DeployMutipleProjects_Release()
            {
                BuildDeployAndCheckDeployment(MethodInfo.GetCurrentMethod().Name, "Release", "Any CPU");
            }
        }

        [TestFixture]
        public class WhidbeyVBDeploymentTests : AbstractPackageOutputsTaskTests
        {
            public override Dictionary<string, WebMap> WebMaps
            {
                get
                {
                    // WebMaps not used in 2.0 web projects
                    return null;
                }
            }

            public WhidbeyVBDeploymentTests()
                : base
                    (
                        @"..\..\..\TestArtifacts\SampleApplications\v2.0\VB\SolutionWithMultipleDifferingProjects.sln",
                        @"..\..\..\TestArtifacts\BuildDrops\v2.0\VB\SolutionWithMultipleDifferingProjects\",
                        DotNetFrameworkVersion.v2_0
                    ) { }

            [Test]
            public void DeployMutipleProjects_Debug()
            {
                BuildDeployAndCheckDeployment(MethodInfo.GetCurrentMethod().Name, "Debug", "Any CPU");
            }

            [Test]
            public void DeployMutipleProjects_Release()
            {
                BuildDeployAndCheckDeployment(MethodInfo.GetCurrentMethod().Name, "Release", "Any CPU");
            }
        }
    }
}