using System;
using System.Collections.Generic;
using System.Text;

using NAnt.VSNet.Types;
using Tests.NAnt.Contrib;
using NUnit.Framework;
using System.IO;
using System.Text.RegularExpressions;

namespace Snak.FunctionalTests
{
    [TestFixture, Category("IntegrationTest")]
    public class SetAssemblyVersionTaskTests : BuildTestBase
    {
        [Test]
        public void SetAssemblyVersionTest()
        {
            List<TestAssemblyVersionData> testData = new List<TestAssemblyVersionData>();

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""1.0.*"")", // actual 
                                            @"AssemblyVersion(""1.0.12.123"")", // expected
                                            null, null, "12", "123")); // values the task will set for each of the build label increments (nulls should not be set)

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""1.0.2.*"")", // actual 
                                            @"AssemblyVersion(""1.0.12.123"")", // expected
                                            null, null, "12", "123")); // values the task will set for each of the build label increments (nulls should not be set)

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""1.0.1.1"")", // actual 
                                            @"AssemblyVersion(""2.0.12.1"")", // expected
                                            "2", null, "12", null)); // values the task will set for each of the build label increments (nulls should not be set)

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""1.0.12.321"")", // actual
                                            @"AssemblyVersion(""1.32.14422.123"")", // expected
                                            "1", "32", "14422", "123")); // values the task will set for each of the build label increments (nulls should not be set)

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""121.10.5.*"")", // actual
                                            @"AssemblyVersion(""121.10.5.19"")", // expected
                                            null, null, null, "19")); // values the task will set for each of the build label increments (nulls should not be set)

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""112.3.0.123"")", // actual
                                            @"AssemblyVersion(""112.3.0.123"")", // expected
                                            null, null, null, null)); // values the task will set for each of the build label increments (nulls should not be set)

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""21.3.*"")", // actual
                                            @"AssemblyVersion(""0.1.381.0"")", // expected
                                            "0", "1", "381", "0")); // values the task will set for each of the build label increments (nulls should not be set)

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""1.33.*"")", // actual
                                            @"AssemblyVersion(""1.33.0.123"")", // note we expect it to put a 0 in the revision number
                                            null, null, null, "123")); // values the task will set for each of the build label increments (nulls should not be set)

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""1.43.*"")", // actual
                                            @"AssemblyVersion(""1.43.0.0"")", // note we expect it to put a 0 in the revision and build numbers
                                            null, null, null, null)); // values the task will set for each of the build label increments (nulls should not be set)

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""1.63.*"")", // actual
                                            @"AssemblyVersion(""3.5.0.0"")", // note we expect it to put a 0 in the revision and build numbers
                                            "3", "5", null, null)); // values the task will set for each of the build label increments (nulls should not be set)

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""1.63.*"")", // actual
                                            @"AssemblyVersion(""1.5.0.0"")",
                                            null, "5", null, null)); // values the task will set for each of the build label increments (nulls should not be set)

            testData.Add(new TestAssemblyVersionData(
                                            @"AssemblyVersion(""1.63.*"")", // actual
                                            @"AssemblyVersion(""1.5.0.5"")",
                                            null, "5", null, "5")); // values the task will set for each of the build label increments (nulls should not be set)        

            try
            {
                string pattern = @"AssemblyVersion\(""(?<buildLabel>[^""]*)""\)";

                string sampleVBFile = File.ReadAllText(@"..\..\..\TestArtifacts\SampleApplications\v2.0\VB\ClassLibrary\ClassLibrary1\My Project\AssemblyInfo.vb");
                string sampleCSharpFile = File.ReadAllText(@"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\WindowsService\WindowsService1\Properties\AssemblyInfo.cs");

                for (int i = 0; i < testData.Count; i++)
                {
                    string fileContents = (i % 2 == 0) ? sampleVBFile : sampleCSharpFile;

                    File.WriteAllText(testData[i].FilePath, Regex.Replace(fileContents, pattern, delegate(Match match) { return testData[i].ActualAssemblyVersion; }));

                    string xmlsnippet = String.Format(
                                            @"<project name=""setAssemblyVersionTaskTests"">
                                                    <loadtasks assembly=""{0}"" />
                                                    <setAssemblyVersion
                                                        {1}
                                                        {2}
                                                        {3}
                                                        {4}>
				                                        <items>
                                                            <include name=""{5}""/>
				                                        </items>
                                                    </setAssemblyVersion>
                                                  </project>",
                                            Path.GetFullPath("Snak.dll"),
                                            (testData[i].VersionToSet.Major != null) ? String.Format(@"major=""{0}"" ", testData[i].VersionToSet.Major) : String.Empty,
                                            (testData[i].VersionToSet.Minor != null) ? String.Format(@"minor=""{0}"" ", testData[i].VersionToSet.Minor) : String.Empty,
                                            (testData[i].VersionToSet.Build != null) ? String.Format(@"build=""{0}"" ", testData[i].VersionToSet.Build) : String.Empty,
                                            (testData[i].VersionToSet.Revision != null) ? String.Format(@"revision=""{0}"" ", testData[i].VersionToSet.Revision) : String.Empty,
                                            testData[i].FilePath
                                        );

                    RunBuild(xmlsnippet);

                    string newFileContents = File.ReadAllText(testData[i].FilePath);

                    for (Match match = Regex.Match(newFileContents, pattern, RegexOptions.IgnoreCase); match.Success; match = match.NextMatch())
                    {
                        if (String.Compare(match.Value, testData[i].ExceptedAssemblyVersion, true) != 0)
                        {
                            Assert.Fail("Incorrect AssemblyVersion replacement: {0}Original:{1}{0}Expected:{2}{0}Actual{3}{0}", Environment.NewLine, testData[i].ActualAssemblyVersion, testData[i].ExceptedAssemblyVersion, match.Value);
                        }
                        else
                        {
                            Console.WriteLine("Correctly set AssemblyVersion: {0}Original:{1}{0}Expected:{2}{0}Actual{3}{0}", Environment.NewLine, testData[i].ActualAssemblyVersion, testData[i].ExceptedAssemblyVersion, match.Value);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex);

                foreach (TestAssemblyVersionData testAssemblyVersionData in testData)
                {
                    File.Delete(testAssemblyVersionData.FilePath);
                }

                throw;
            }
        }

        private class TestAssemblyVersionData
        {
            public readonly string FilePath = String.Empty;
            public readonly string ActualAssemblyVersion = String.Empty;
            public readonly string ExceptedAssemblyVersion = String.Empty;
            public readonly AssemblyVersionToSet VersionToSet = null;

            /// <summary>
            /// 
            /// </summary>
            /// <param name="actualAssemblyVersion">The assembly version that will get injected into an assembly info file and with then be manipulated by the code under test.</param>
            /// <param name="majorToSet">the major increment that you want to set against the actualAssemblyVersion param. Set null if you do not want this changed.</param>
            /// <param name="monorToSet">the minor increment that you want to set against the actualAssemblyVersion param. Set null if you do not want this changed.</param>
            /// <param name="buildToSet">the build increment that you want to set against the actualAssemblyVersion param. Set null if you do not want this changed.</param>
            /// <param name="revisionToSet">the revision increment that you want to set against the actualAssemblyVersion param. Set null if you do not want this changed.</param>
            public TestAssemblyVersionData(string actualAssemblyVersion, string exceptedAssemblyVersion, string majorToSet, string minorToSet, string buildToSet, string revisionToSet)
            {
                FilePath = Path.GetTempFileName();
                ActualAssemblyVersion = actualAssemblyVersion;
                ExceptedAssemblyVersion = exceptedAssemblyVersion;
                VersionToSet = new AssemblyVersionToSet(actualAssemblyVersion, majorToSet, minorToSet, buildToSet, revisionToSet);
            }

            public class AssemblyVersionToSet
            {
                public string Major = String.Empty;
                public string Minor = String.Empty;
                public string Build = String.Empty;
                public string Revision = String.Empty;

                public AssemblyVersionToSet(string actualAssemblyVersion, string major, string minor, string build, string revision)
                {
                    Major = major;
                    Minor = minor;
                    Build = build;
                    Revision = revision;
                }
            }
        }
    }
}