using System;
using System.IO;
using System.Collections.Generic;
using System.Text;

using NAnt.VSNet.Types;
using Tests.NAnt.Contrib;
using NUnit.Framework;

using Snak.Core;
using Snak.BuildDrop;

namespace Snak.FunctionalTests
{
    public class DeploySolutionTaskTests 
    {
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //
        // .Net v1.1 deployment tests (Everett)
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        [TestFixture]
        public class EverettCSharpeDeploymentTests : AbstractDeploySolutionTaskTest
        {
            private Dictionary<string, WebMap> _webMaps = null;

            public override Dictionary<string, WebMap> WebMaps
            {
              get 
              {
                  if (_webMaps == null)
                  {
                    _webMaps = new Dictionary<string, WebMap>();
                    WebMap map = new WebMap();
                    map.Url = "http://localhost/WebApplication1/WebApplication1.csproj";
                    map.Path = new FileInfo(Path.GetFullPath(@"..\..\..\TestArtifacts\SampleApplications\v1.1\C#\AspNetWebApplication\WebApplication1\WebApplication1.csproj"));
                    _webMaps.Add("WebApplication1",map);
                    map = new WebMap();
                    map.Url = "http://localhost/WebService1/WebService1.csproj";
                    map.Path = new FileInfo(Path.GetFullPath(@"..\..\..\TestArtifacts\SampleApplications\v1.1\C#\AspNetWebApplication\WebService1\WebService1.csproj"));
                    _webMaps.Add("WebService1", map);
                  }

                  return _webMaps; 
              }
            }

            public EverettCSharpeDeploymentTests()
                : base
                (
                    @"..\..\..\TestArtifacts\SampleApplications\v1.1\C#\SolutionWithMultipleDifferingProjects.sln",
                    @"..\..\..\TestArtifacts\BuildDrops\v1.1\C#\SolutionWithMultipleDifferingProjects\",
                    DotNetFrameworkVersion.v1_1
                ) { }

            [Test]
            public void DeployMutipleProjects_Debug()
            {
                BuildDeployAndCheckDeployment("Debug", false);
            }

            [Test]
            public void DeployMutipleProjects_Debug_CreateVDirs()
            {
                BuildDeployAndCheckDeployment("Debug", true);
            }

            [Test]
            public void DeployMutipleProjects_Release()
            {
                BuildDeployAndCheckDeployment("Release", false);
            }

            [Test]
            public void DeployMutipleProjects_Release_CreateVDirs()
            {
                BuildDeployAndCheckDeployment("Release", true);
            }
        }

        [TestFixture]
        public class EverettVBDeploymentTests : AbstractDeploySolutionTaskTest
        {
            private Dictionary<string, WebMap> _webMaps = null;

            public override Dictionary<string, WebMap> WebMaps
            {
              get 
              {
                  if (_webMaps == null)
                  {
                      _webMaps = new Dictionary<string, WebMap>();
                      WebMap map = new WebMap();
                      map.Url = "http://localhost/WebApplication1/WebApplication1.vbproj";
                      map.Path = new FileInfo(Path.GetFullPath(@"..\..\..\TestArtifacts\SampleApplications\v1.1\VB\AspNetWebApplication\WebApplication1\WebApplication1.vbproj"));
                      _webMaps.Add("WebApplication1", map);
                      map = new WebMap();
                      map.Url = "http://localhost/WebService1/WebService1.vbproj";
                      map.Path = new FileInfo(Path.GetFullPath(@"..\..\..\TestArtifacts\SampleApplications\v1.1\VB\AspNetWebApplication\WebService1\WebService1.vbproj"));
                      _webMaps.Add("WebService1", map);
                  }

                  return _webMaps; 
              }
            }

            public EverettVBDeploymentTests() 
                : base
                (
                    @"..\..\..\TestArtifacts\SampleApplications\v1.1\VB\SolutionWithMultipleDifferingProjects.sln", 
                    @"..\..\..\TestArtifacts\BuildDrops\v1.1\VB\SolutionWithMultipleDifferingProjects\", 
                    DotNetFrameworkVersion.v1_1
                ) { }

            [Test]
            public void DeployMutipleProjects_Debug()
            {
                BuildDeployAndCheckDeployment("Debug", false);
            }

            [Test]
            public void DeployMutipleProjects_Debug_CreateVDirs()
            {
                BuildDeployAndCheckDeployment("Debug", true);
            }

            [Test]
            public void DeployMutipleProjects_Release()
            {
                BuildDeployAndCheckDeployment("Release", false);
            }

            [Test]
            public void DeployMutipleProjects_Release_CreateVDirs()
            {
                BuildDeployAndCheckDeployment("Release", true);
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //
        // .Net v2.0 deployment tests (Whidbey)
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        [TestFixture]
        public class WhidbeyCSharpeDeploymentTests : AbstractDeploySolutionTaskTest 
        {
            public override Dictionary<string, WebMap> WebMaps
            {
                get 
                {  
                    // WebMaps not used in 2.0 web projects
                    return null ; 
                }
            }

            public WhidbeyCSharpeDeploymentTests() 
                : base
                (
                    @"..\..\..\TestArtifacts\SampleApplications\v2.0\C#\SolutionWithMultipleDifferingProjects.sln", 
                    @"..\..\..\TestArtifacts\BuildDrops\v2.0\C#\SolutionWithMultipleDifferingProjects\", 
                    DotNetFrameworkVersion.v2_0
                ) { }

            [Test]
            public void DeployMutipleProjects_Debug()
            {
                BuildDeployAndCheckDeployment("Debug", "Any CPU", false);
            }

            [Test]
            public void DeployMutipleProjects_Debug_CreateVDirs()
            {
                BuildDeployAndCheckDeployment("Debug", "Any CPU", true);
            }

            [Test]
            public void DeployMutipleProjects_Release()
            {
                BuildDeployAndCheckDeployment("Release", "Any CPU", false);
            }

            [Test]
            public void DeployMutipleProjects_Release_CreateVDirs()
            {
                BuildDeployAndCheckDeployment("Release", "Any CPU", true);
            }
        }

        [TestFixture]
        public class WhidbeyVBDeploymentTests : AbstractDeploySolutionTaskTest
        {
            public override Dictionary<string, WebMap> WebMaps
            {
                get
                {
                    // WebMaps not used in 2.0 web projects
                    return null;
                }
            }

            public WhidbeyVBDeploymentTests() 
                    : base
                    (
                        @"..\..\..\TestArtifacts\SampleApplications\v2.0\VB\SolutionWithMultipleDifferingProjects.sln", 
                        @"..\..\..\TestArtifacts\BuildDrops\v2.0\VB\SolutionWithMultipleDifferingProjects\", 
                        DotNetFrameworkVersion.v2_0
                    ) { }

            [Test]
            public void DeployMutipleProjects_Debug()
            {
                BuildDeployAndCheckDeployment("Debug", "Any CPU", false);
            }

            [Test]
            public void DeployMutipleProjects_Debug_CreateVDirs()
            {
                BuildDeployAndCheckDeployment("Debug", "Any CPU", true);
            }

            [Test]
            public void DeployMutipleProjects_Release()
            {
                BuildDeployAndCheckDeployment("Release", "Any CPU", false);
            }

            [Test]
            public void DeployMutipleProjects_Release_CreateVDirs()
            {
                BuildDeployAndCheckDeployment("Release", "Any CPU", true);
            }
        }
    }
}