using System;
using System.Text.RegularExpressions;
using System.Collections.Generic;
using System.Text;

using NAnt.Core;

namespace Snak.Core
{
    public class BuildLabel
    {
        private Version _version;
        private BuildLabelType _buildLabelType = BuildLabelType.Numeric;
        private string _buildLabelString;

        public Version Version
        {
            get
            {
                if (_buildLabelType != BuildLabelType.VersionNumber)
                    throw new InvalidOperationException("The property '" + System.Reflection.MethodInfo.GetCurrentMethod().Name + "' is only accessible when '" + this.GetType().ToString() + ".BuildLabelType' is set to VersionNumber. This property is set internally, the likely cause of the problem is you are not specifying a valid version label.");

                return _version;
            }
        }
        public int BuildLabelNumber
        {
            get
            {
                if (this._buildLabelType != BuildLabelType.Numeric)
                    throw new InvalidOperationException("The property '" + System.Reflection.MethodInfo.GetCurrentMethod().Name + "' is only accessible when '" + this.GetType().ToString() + ".BuildLabelType' is set to Numeric. This property is set internally, the likely cause of the problem is you are not specifying a valid numerical build label.");
                else
                    return int.Parse(this._buildLabelString);
            }
        }












        public BuildLabelType BuildLabelType
        {
            get { return _buildLabelType; }
        }

        public string BuildLabelString
        {
            get
            {
                if (this._buildLabelType == BuildLabelType.VersionNumber)
                    return _version.ToString();
                else
                    return _buildLabelString;
            }
        }

        public BuildLabel(string buildLabel)
        {
            _buildLabelString = buildLabel;
            Init();
        }

        private void Init()
        {
            if (_buildLabelString != String.Empty)
            {
                string buildLabelFormatErrorMessage = "The value for the attribute 'buildLabel' must be either numeric (e.g. 156) or else a version number (e.g. 1.0.0.0).";

                if (Regex.Match(_buildLabelString, @"[^0-9|\.]").Success)
                {
                    throw new BuildException(buildLabelFormatErrorMessage);
                }

                try
                {
                    string[] versionParts = _buildLabelString.Split('.');

                    if (versionParts.Length > 1 && versionParts.Length < 5)
                    {
                        _buildLabelType = BuildLabelType.VersionNumber;

                        int[] versionPartNumbers = new int[versionParts.Length];

                        for (int i = 0; i < versionParts.Length; i++)
                            versionPartNumbers[i] = versionPartNumbers[i];

                        switch (versionPartNumbers.Length)
                        {
                            case 2:
                                _version = new Version(versionPartNumbers[0], versionPartNumbers[1]);
                                break;
                            case 3:
                                _version = new Version(versionPartNumbers[0], versionPartNumbers[1], versionPartNumbers[3]);
                                break;
                            case 4:
                                _version = new Version(versionPartNumbers[0], versionPartNumbers[1], versionPartNumbers[3], versionPartNumbers[4]);
                                break;
                            default:
                                throw new BuildException(buildLabelFormatErrorMessage);
                        }
                    }
                    else if (versionParts.Length == 1)
                    {
                        _buildLabelType = BuildLabelType.Numeric;
                    }
                    else
                        throw new BuildException(buildLabelFormatErrorMessage);
                }
                catch (ArgumentException)
                {
                    // the Version class throws ArgumentException s, we could catch them here an give a more informative message to the user
                    // but the code above checking what we pass to the Version class so we dont really need to catch anything here...
                    // throw new BuildException("", ex);
                    throw;
                }
            }
            else
            {
                _buildLabelString = "0";
                _buildLabelType = BuildLabelType.Numeric;
            }
        }
    }
}