using System;
using System.Collections.Generic;
using System.Text;

using Snak.Core;

namespace Snak.Core
{
    /// <summary>
    /// This class contains data similar to that found in the visual studio configuration manager including the 
    /// state of the active build configuration, when this changes the delegate provide during instantiation is called.
    /// </summary>
    /// <exception cref="SnakException">a SnakException occurs is the delegate provided to the constructor is null.</exception>
    public class ConfigurationManagerState
    {
        public delegate void ReloadOnConfigurationChangeDelegate();

        ReloadOnConfigurationChangeDelegate _reloadOnConfigurationChangeDelegate = null;

        private Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>> _solutionsProjectConfigsKeyedOnSolutionsBuildConfig;

        public Dictionary<BuildConfiguration, Dictionary<Guid, SolutionsProjectConfig>> SolutionsProjectConfigsKeyedOnSolutionsBuildConfig
        {
            get { return _solutionsProjectConfigsKeyedOnSolutionsBuildConfig; }
            set { _solutionsProjectConfigsKeyedOnSolutionsBuildConfig = value; }
        }

        /// <summary>
        /// Gets the solutions configurations based on the ActiveBuildConfiguration, note that this will return a configuration for each project,
        /// </summary>
        /// <returns></returns>
        /// <remarks>refer to the SolutionsProjectConfig.Build property to determine if a the given project needs to be built.</remarks>
        public Dictionary<Guid, SolutionsProjectConfig> GetSolutionsProjectConfigsBasedOnActiveConfiguration()
        {
            return this.SolutionsProjectConfigsKeyedOnSolutionsBuildConfig[this.ActiveBuildConfiguration];
        }

        private BuildConfiguration _activeBuildConfiguration = null;

        public BuildConfiguration ActiveBuildConfiguration
        {
            get { return (this._activeBuildConfiguration); }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="activeBuildConfiguration"></param>
        /// <param name="reloadOnConfigurationChangeDelegate">this delegate will be called when a configuration change happens</param>
        internal ConfigurationManagerState(BuildConfiguration activeBuildConfiguration, ReloadOnConfigurationChangeDelegate reloadOnConfigurationChangeDelegate)
        {
            if (reloadOnConfigurationChangeDelegate == null)
                throw new ArgumentException("reloadOnConfigurationChangeDelegate");

            this._activeBuildConfiguration = activeBuildConfiguration; // we allow this to be a null reff if null is passed in
            this._reloadOnConfigurationChangeDelegate = reloadOnConfigurationChangeDelegate;
        }

        /// <summary>
        /// Changes the active build configuration and calls a reload delegate provided during instantiation of the ConfigurationManagerState.
        /// </summary>
        /// <param name="activeBuildConfiguration"></param>
        public void SetActiveBuildConfigurationAndLoad(BuildConfiguration activeBuildConfiguration)
        {
            if (this._activeBuildConfiguration != activeBuildConfiguration)
            {
                this._activeBuildConfiguration = activeBuildConfiguration;

                if (_reloadOnConfigurationChangeDelegate == null)
                    throw new SnakException("The delegate target responsible for reloading on a configuration change is null.");

                _reloadOnConfigurationChangeDelegate();
            }        
        }
    }
}
