using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace Snak.Core.Everett
{
    internal class EverettXmlProjectFileParser : AbstractXmlProjectFileParser
    {
        XmlElement _projectFileSettings = null;
        XmlElement _projectSettings = null;

        internal EverettXmlProjectFileParser(XmlDocument document, XmlNamespaceManager xmlNamespaceManager)
            : base(document, xmlNamespaceManager)
        {
            _projectSettings = (XmlElement)this.ProjectDocument.SelectSingleNode("//Build/Settings");
            _projectFileSettings = (XmlElement)this.ProjectDocument.SelectSingleNode("/*/*");
        }

        internal override T GetPropertySetting<T>(string propertyName)
        {
            string attributeValue = _projectSettings.GetAttribute(propertyName);

            // if we couldn't get it from the projects 'settings' element we try grab if from the root level project file element
            if (attributeValue == String.Empty)
                attributeValue = _projectFileSettings.GetAttribute(propertyName);

            // if the value is String.Empty then its going to blow up below here, but thats ok... caller can deal with it

            return this.TryConvertValue<T>(attributeValue);
        }

        internal override T GetPropertySettingForBuildConfig<T>(BuildConfiguration buildConfiguration, string propertyName)
        {
            // Note in this next line we only specify the 'buildConfiguration.Config' as apposed to buildConfiguration.ToString().
            // it's bizarrer but the configuration manager in 2003 does have a platform setting for the projects (this is represented in the solution file), 
            // therefore the buildConfiguration object  passed to this method should have both a Config and a Platform value and its ToString method 
            // will return something like 'Debug|.Net'. However the actual project file does not have any platform specific settings.
            // Perhaps Im missing something with how 2003 works, but until it jumps out at me Im omitting the platform setting from the xpath below in 
            // order to correctly pluck the correct configuration information from the project file
            string findBuildConfigXPath = string.Format("//Build/Settings/Config[@Name='{0}']", buildConfiguration.Config);

            XmlElement configSettings = (XmlElement)this.ProjectDocument.SelectSingleNode(findBuildConfigXPath);

            string noteValue = configSettings.GetAttribute(propertyName);

            return this.TryConvertValue<T>(noteValue);
        }
    }
}
