using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Snak.Core
{
    public interface IProjectProperties
    {
        /// <summary>
        /// The name of the project
        /// </summary>
        string ProjectName { get; }

        /// <summary>
        /// The safe name of the project, useful for using for directory names
        /// </summary>
        string ProjectSafeName { get; }

        /// <summary>
        /// 
        /// </summary>
        Guid ID { get; }

        /// <summary>
        /// 
        /// </summary>
        FileInfo ProjectFile { get; }

        /// <summary>
        /// The project file's file extension (csproj / vbproj)
        /// </summary>
        string ProjectFileExtension { get; }

        /// <summary>
        /// The file extension for code files for this project type (eg .cs 
        /// </summary>
        string CodeFileExtension { get; }

        /// <summary>
        /// The path to the project root
        /// </summary>
        DirectoryInfo ProjectDir { get; }

        /// <summary>
        /// The language that the project is written in
        /// </summary>
        ProjectLanguage ProjectLanguage { get; }

        /// <summary>
        /// The (simple) name of the project's output assembly (no file extension)
        /// </summary>
        string AssemblyName { get; }

        /// <summary>
        /// The name of the project's output assembly with the file extension
        /// </summary>
        string AssemblyFileName { get; }

        /// <summary>
        /// The type of project it is (dll / exe)
        /// </summary>
        ProjectOutputType ProjectOutputType { get; }

        /// <summary>
        /// Determines the type of the project as read from the project files ProjectType element
        /// </summary>
        ProjectType ProjectType { get; }

        /// <summary>
        /// The project's configuration file (web.config or app.config)
        /// </summary>
        string ConfigFile { get; }

        /// <summary>
        /// Gets an array of all the files contained within the project
        /// All paths are project-relative
        /// </summary>
        List<String> GetProjectFiles();

        /// <summary>
        /// Gets an array of all the files contained within the project
        /// that match the content type provided (eg 'Content' for web pages).
        /// All paths are project-relative
        /// </summary>
        List<String> GetProjectFiles(ProjectFileType contentType);

        /// <summary>
        /// The .net framework the project is targeted towards.
        /// </summary>
        DotNetFrameworkVersion DotNetFrameworkVersion { get; }

        /// <summary>
        /// returns an IProjectBuildConfig for the given BuildConfiguration
        /// </summary>
        // Dictionary<BuildConfiguration, IProjectBuildConfig> ProjectBuildConfig { get; }  

        // AbstractProjectBuildConfig ActiveAbstractProjectBuildConfig { get; }

        IProjectBuildProperties IProjectBuildProperties { get; }
    }
}
