using System;
using System.Collections.Generic;
using System.Text;

namespace Snak.Core
{
    internal class ProjectTypeFactory
    {
        // Project types:
        // http://msdn2.microsoft.com/en-us/library/hb23x61k.aspx suggests here HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\VisualStudio\8.0\Policy\SupportedProjects.
        // there are also more listed here HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\VisualStudio\8.0\Projects
        //
        // {FAE04EC0-301F-11d3-BF4B-00C04F79EFBC} csproj
        // {F184B08F-C81C-45f6-A57F-5ABD9991F28F} vbproj
        // {978C614F-708E-4E1A-B201-565925725DBA} Visual Studio Deployment Setup Project
        // {8BC9CEB9-8B4A-11D0-8D11-00A0C91BC942} Exe Projects
        // {54435603-DBB4-11D2-8724-00A0C9A8B90C} Visual Studio Deployment Project
        // {4fd007e8-1a56-7e75-70ca-0466484d4f98} VisualBasic Test Project

        // {3AC096D0-A1C2-E12C-1390-A8335801FDAB} Test Project
        // {39d444fd-b490-1554-5274-2d612a165298} CSharp Test Project
        // {349c5851-65df-11da-9384-00065b846f21} Web Application Project Factory (Web application project)

        static Guid solutionFolderProject = new Guid("66A26720-8FB5-11D2-AA7E-00C04F688DDE");
        static Guid vbproj = new Guid("F184B08F-C81C-45F6-A57F-5ABD9991F28F");
        static Guid csproj = new Guid("FAE04EC0-301F-11D3-BF4B-00C04F79EFBC");
        static Guid webSite = new Guid("E24C65DC-7377-472b-9ABA-BC803B73C61A");

        static Guid visualStudioDeploymentSetupProject = new Guid("978C614F-708E-4E1A-B201-565925725DBA");
        static Guid exeProjects = new Guid("8BC9CEB9-8B4A-11D0-8D11-00A0C91BC942");
        static Guid visualStudioDeploymentProject = new Guid("54435603-DBB4-11D2-8724-00A0C9A8B90C");
        static Guid visualBasicTestProject = new Guid("4fd007e8-1a56-7e75-70ca-0466484d4f98");
        
        static Guid testProject = new Guid("3AC096D0-A1C2-E12C-1390-A8335801FDAB");
        static Guid cSharpTestProject = new Guid("39d444fd-b490-1554-5274-2d612a165298");
        static Guid webApplicationProject = new Guid("349c5851-65df-11da-9384-00065b846f21");

        public static ProjectType GetProjectType(Guid projectTypeGuid)
        {
            ProjectType projectTypeToReturn = ProjectType.Unknown; // default value

            if (projectTypeGuid.Equals(solutionFolderProject))
                projectTypeToReturn = ProjectType.SolutionFolder;
            else if (projectTypeGuid.Equals(vbproj))
                projectTypeToReturn = ProjectType.VisualBasic;
            else if (projectTypeGuid.Equals(csproj))
                projectTypeToReturn = ProjectType.VisualCSharp;
            else if (projectTypeGuid.Equals(webSite))
                projectTypeToReturn = ProjectType.WebSite;

            if (projectTypeGuid.Equals(visualStudioDeploymentSetupProject))
                projectTypeToReturn = ProjectType.VisualStudioDeploymentSetup;
            else if (projectTypeGuid.Equals(exeProjects))
                projectTypeToReturn = ProjectType.Exe;
            else if (projectTypeGuid.Equals(visualStudioDeploymentProject))
                projectTypeToReturn = ProjectType.VisualStudioDeployment;
            else if (projectTypeGuid.Equals(visualBasicTestProject))
                projectTypeToReturn = ProjectType.VisualBasicTest;

            if (projectTypeGuid.Equals(testProject))
                projectTypeToReturn = ProjectType.Test;
            else if (projectTypeGuid.Equals(cSharpTestProject))
                projectTypeToReturn = ProjectType.CSharpTest;
            else if (projectTypeGuid.Equals(webApplicationProject))
                projectTypeToReturn = ProjectType.WebApplication;

            return projectTypeToReturn;
        }

        public static ProjectType GetProjectType(string projectTypeString)
        {
            if (projectTypeString == null)
                throw new ArgumentNullException("projectTypeString");
            
            if (projectTypeString == String.Empty)
                throw new ArgumentException("projectTypeString cannot be empty");

            ProjectType projectType = ProjectType.Unknown;

            try
            {
                projectType = (ProjectType)Enum.Parse(typeof(ProjectType), projectTypeString, true);
            }
            catch (ArgumentException)
            {
                // TODO: log

                // we dont want to blow up here we just want to return the default
            }

            return projectType;
        }
    }
}
