using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace Snak.Core.Whidbey
{
    internal abstract class WhidbeyCommonProjectInfo : AbstractProject
    {
        internal WhidbeyCommonProjectInfo(FileInfo projectFile, BuildConfiguration buildConfigurationToLoad)
            : base(projectFile, buildConfigurationToLoad)
        {

        }

        internal WhidbeyCommonProjectInfo(Guid id, string projectName, FileInfo projectFile, BuildConfiguration buildConfigurationToLoad)
            : base(id, projectName, projectFile, buildConfigurationToLoad)
        {

        }

        internal WhidbeyCommonProjectInfo(Guid id, string projectName, string solutionsProjectSection, DirectoryInfo projectDir, BuildConfiguration buildConfigurationToLoad)
            : base(id, projectName, solutionsProjectSection, projectDir, buildConfigurationToLoad)
        {

        }

        protected abstract XmlNamespaceManager GetXmlXMLNamespaceManagerForProjectFile();

        protected abstract void LoadNonBuildConfigSpeificProperties()

        protected abstract void LoadNonBuildConfigSpeificProperties_ForFilelessProjects();

        protected abstract void LoadProjectSettingsForBuildConfig(BuildConfiguration buildConfiguration);

        protected abstract AbstractXmlProjectFileParser GetAbstractXmlProjectFileParser(XmlDocument projectDocument, XmlNamespaceManager xmlNamespaceManager);

        public abstract List<string> GetProjectFiles();

        /// <summary>
        /// returns the projects files according to the content type, if the project is a fileless project e.g. a 2005 web site then all file are returned. 
        /// </summary>
        /// <param name="contentType"></param>
        /// <returns></returns>
        public abstract List<string> GetProjectFiles(ProjectFileType contentType);

        protected ProjectType GetProjectType()
        {
            ProjectType projectType = ProjectType.Unknown;
            string projectTypeString;

            // there seems to be some inconsistencies regarding how to determine the 'project type' given the project file
            // some project files have a ProjectType element, some dont have a ProjectType but have a ProjectTypeGuids element.
            // Some such as the .wdproj dont have either of the above mentioned.
            //
            // Given this we end up with a few areas were we can figure out the project type. At this point we really only care about
            // Web projects, Deployment projects and others (which we will label Local). (In 2003 we had Web or Local).
            //
            // At this point we can get the info from one of 3 places.
            //
            // 1: for project file that have a ProjectType element we can simply get that value, if that fails
            //
            // 2: for project file that have a ProjectTypeGuids element we can look in the registry, 
            // we could sniff around in the registry using code here but we have a class ProjectTypeFactory that has some hardcoded Guids, yuk.
            // that'll do for now.
            //
            // 3: for project file that dont have ProjectType or ProjectTypeGuids element we can look at the file extension
            //
            // 4: if that fails we will just give it a ProjectType ProjectType.Unknown

            projectTypeString = this.XmlProjectDocParser.GetPropertySetting<String>("ProjectType");

            if (projectTypeString == String.Empty)
            {
                string guid = String.Empty;
                string guids = this.XmlProjectDocParser.GetPropertySetting<String>("ProjectTypeGuids");

                if (guids != null && guids != String.Empty)
                {
                    if (guids.IndexOf(";", 0, guids.Length) > -1)
                        guid = guids.Split(';')[0];
                    else
                        guid = guids;

                    projectType = ProjectTypeFactory.GetProjectType(new Guid(guid));
                }

                if (projectType == ProjectType.Unknown && this.ProjectFile.Extension == ".wdproj")
                    projectType = ProjectType.VisualStudioDeployment;

                if (projectType == ProjectType.Unknown)
                {
                    // TODO: figure out how the nant logging working in here
                    // or perhaps configure log4 net and globally try point it to nant output
                    Console.WriteLine("Could not find a project type for '" + this.ProjectFile.FullName + "' using default value of 'ProjectType.Unknown'");
                }
            }
            else
            {
                projectType = ProjectTypeFactory.GetProjectType(projectTypeString);
            }

            return projectType;
        }
    }
}
