using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;
using System.Xml;

namespace Snak.Core.Whidbey
{
    public class WhidbeyWebProjectInfo : AbstractProject
    {
        public override string ProjectSafeName
        {
            get
            {
                return Regex.Replace(this.ProjectName, @"([a-z]{1}:\\){1}|[\.|\\]*", "", RegexOptions.IgnoreCase);// removes . and \ and : from the project name
            }
        }

        internal WhidbeyWebProjectInfo(Guid id, string projectName, string solutionsProjectSection, DirectoryInfo projectDir, BuildConfiguration buildConfigurationToLoad)
            : base(id, projectName, solutionsProjectSection, projectDir, buildConfigurationToLoad)
        {

        }

        protected override XmlNamespaceManager GetXmlXMLNamespaceManagerForProjectFile()
        {
            throw new NotSupportedException();
        }

        protected override void LoadNonBuildConfigSpeificProperties_ForFilelessProjects()
        {
            this.ProjectLanguage = (this.SolutionsProjectSection.Contains(@"DefaultWebSiteLanguage = ""Visual C#""")) ? ProjectLanguage.CSHARP : ProjectLanguage.VisualBasic;
            this.AssemblyName = String.Empty;
            this.ProjectType = ProjectType.WebSite;
            this.DotNetFrameworkVersion = DotNetFrameworkVersion.v2_0;

            if (this.ID == Guid.Empty)
                throw new SnakException("The Project ID should have been set allready.");

            this.ProjectOutputType = ProjectOutputType.Library;
        }

        public override List<string> GetProjectFiles()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// returns the projects files according to the content type, if the project is a fileless project e.g. a 2005 web site then all file are returned. 
        /// </summary>
        /// <param name="contentType"></param>
        /// <returns></returns>
        public override List<string> GetProjectFiles(ProjectFileType contentType)
        {
            List<string> files = null;

            files = new List<string>((IEnumerable<string>)Directory.GetFiles(this.ProjectDir.FullName));

            for (int i = 0; i < files.Count; i++)
                files[i] = Path.GetFileName(files[i]);

            return files;
        }

        protected override void LoadProjectSettingsForBuildConfig(BuildConfiguration buildConfiguration)
        {
            AbstractProjectBuildProperties abstractProjectBuildConfig = null;

            abstractProjectBuildConfig = new WhidbeyWebProjectBuildConfig(buildConfiguration, this, this.SolutionsProjectSection);

            this.IProjectBuildProperties = abstractProjectBuildConfig;
        }

        protected override AbstractXmlProjectFileParser GetAbstractXmlProjectFileParser(XmlDocument projectDocument, XmlNamespaceManager xmlNamespaceManager)
        {
            throw new InvalidOperationException();
        }

        protected override void LoadNonBuildConfigSpeificProperties()
        {
            throw new NotImplementedException();
        }
    }
}
