using System.IO;
using NAnt.Core;
using NAnt.Core.Attributes;

using Snak.Core;

namespace Snak.Tasks
{
	/// <summary>
	/// A base class for NAnt tasks that work with Visual Studio project's
	/// </summary>
	public abstract class ProjectTask : Task
	{
		private FileInfo _projectFile;
		private string _config;
        private IProjectInfo _iProjectInfo = null;

        internal IProjectInfo IProjectInfo
        {
            get { return _iProjectInfo; }
            set { _iProjectInfo = value; }
        }

		/// <summary>
		/// Specifies the project which is being manipulated
		/// </summary>
		[TaskAttribute("project", Required=true)]
		public FileInfo ProjectFile {
			get { return _projectFile; }
			set { _projectFile = value; }
		}

		/// <summary>
		/// Specifies the project's build configuration that's being used
		/// </summary>
		[TaskAttribute("config", Required=true)]
		[StringValidator(AllowEmpty=false)]
		public string Config {
			get { return _config; }
			set { _config = value; }
		}

		protected IProjectInfo GetProject()
		{
            if (this._iProjectInfo == null)
                this._iProjectInfo = ProjectFactory.GetProject(_projectFile, new BuildConfiguration(_config));

            return this._iProjectInfo;
		}
	}
}