using System;
using System.IO;
using NAnt.Core;
using NAnt.Core.Attributes;

using Snak.ConfigurationTransformation;

namespace Snak.Tasks
{
	/// <summary>
	/// Summary description for ConfigurationFileTransformer.
	/// </summary>
	[TaskName("transformConfigurationFile")]
	public class TransformConfigurationFileTask : Task
	{
		private FileInfo _targetConfigurationFile = null;
		private FileInfo _outputConfigurationFile = null;
		private string _targetEnvironment = String.Empty;

		[TaskAttribute("targetConfigurationFile", Required=true)]
		public FileInfo TargetConfigurationFile
		{
			get { return _targetConfigurationFile; }
			set { _targetConfigurationFile = value; }
		}

		[TaskAttribute("outputConfigurationFile", Required=true)]
		public FileInfo OutputConfigurationFile
		{
			get { return _outputConfigurationFile; }
			set { _outputConfigurationFile = value; }
		}

		[TaskAttribute("targetEnvironment", Required=true)]
		public string TargetEnvironment
		{
			get { return _targetEnvironment; }
			set { _targetEnvironment = value; }
		}

		public TransformConfigurationFileTask()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		protected override void ExecuteTask()
		{
            
			if (!_targetConfigurationFile.Exists)
				throw new ApplicationException("Could not find the target configuration file '" + _targetConfigurationFile.FullName + "'");


			ConfigurationTransformer transformer = new ConfigurationTransformer();
			transformer.Transform(_targetEnvironment, _targetConfigurationFile.FullName, _outputConfigurationFile.FullName);				
		}
	}
}
